{
  "type": "record",
  "Aspect": {
    "name": "dataHubRemoteExecutorGlobalConfigKey",
    "keyForEntity": "dataHubRemoteExecutorGlobalConfig",
    "entityCategory": "internal",
    "entityAspects": [
      "dataHubRemoteExecutorPoolGlobalConfig"
    ]
  },
  "name": "RemoteExecutorGlobalConfigKey",
  "namespace": "com.linkedin.pegasus2avro.metadata.key",
  "fields": [
    {
      "type": "string",
      "name": "id",
      "doc": "The unique identifier for the remote executor global config\nNOTE: since this a singleton, there should be a hardcoded key in the AcrylConstants file"
    }
  ],
  "doc": "Key for the *Singleton* DataHub Remote Executor Global Config"
}